## 一、 docker 拉去最新版本的redis

      `docker pull redis:6.0.6` #后面可以带上tag号, 默认拉取最新版本

## 二、 docker安装`redis`

执行命令:

```sh
mkdir -p /usr/local/docker/redis/conf
chmod -R 777 /usr/local/docker/redis
cd /usr/local/docker/redis/conf
#wget https://github.com/redis/redis/blob/6.0.6/redis.conf #下载redis的配置文件 (注意可跳此步,直接下一步,我已经写好配置了，或者直接打开连接找到对应版本的配置文件下载到服务器上，再按我下面的配置文件修改一下)
vim redis.conf #更改配置文件, 让宿主机可以访问redis-server #保存并退出
```
编辑配置文件
/usr/local/docker/redis/conf/redis.conf

**redis.conf 文件**

```sh
#开启保护
protected-mode yes
# 注释下面一行配置 开启远程连接 
#bind 127.0.0.1 
#自定义密码
requirepass 123456 
port 6379
timeout 0
# 900s内至少一次写操作则执行bgsave进行RDB持久化
save 900 1 
save 300 10
save 60 10000
# rdbcompression ；默认值是yes。对于存储到磁盘中的快照，可以设置是否进行压缩存储。如果是的话，redis会采用LZF算法进行压缩。如果你不想消耗CPU来进行压缩的话，可以设置为关闭此功能，但是存储在磁盘上的快照会比较大。
rdbcompression yes
# dbfilename ：设置快照的文件名，默认是 dump.rdb
dbfilename dump.rdb
# dir：设置快照文件的存放路径，这个配置项一定是个目录，而不能是文件名。使用上面的 dbfilename 作为保存的文件名。
dir /data
# 默认redis使用的是rdb方式持久化，这种方式在许多应用中已经足够用了。但是redis如果中途宕机，会导致可能有几分钟的数据丢失，根据save来策略进行持久化，Append Only File是另一种持久化方式， 可以提供更好的持久化特性。Redis会把每次写入的数据在接收后都写入appendonly.aof文件，每次启动时Redis都会先把这个文件的数据读入内存里，先忽略RDB文件。默认值为no。
appendonly yes
# appendfilename ：aof文件名，默认是"appendonly.aof"
# appendfsync：aof持久化策略的配置；no表示不执行fsync，由操作系统保证数据同步到磁盘，速度最快；always表示每次写入都执行fsync，以保证数据同步到磁盘；everysec表示每秒执行一次fsync，可能会导致丢失这1s数据
appendfsync everysec

```
保存退出

参数说明
1.修改保护模式protected-mode yes 默认为yes 可以跳过这一步

​ Redis protected-mode属性解读

​ 设置外部网络连接redis服务，设置说明如下：

​ a.关闭protected-mode模式，此时外部网络可以直接访问

​ b.开启protected-mode保护模式，需配置bind ip 和设置访问密码 redis3.2版本后新增protected-mode配置，默认是yes，即开启。

2.把bind 127.0.0.1 注释掉 #bind 127.0.0.1,  这样所有的ip都可以访问了

3.设置密码（根据自己的需要）

设置永久密码的方法

找到requirepass foobared 把foobared改为自己的登陆密码 这里我设置为123456

requirepass 12345678

设置临时密码的方法

在连接上redis后,config set设置临时密码,redis重启后,设置的密码就失效了

127.0.0.1:6379> config set requirepass "123456" # 设置密码为12345678
OK
127.0.0.1:6379> config get requirepass # 获取密码的值
1) "requirepass"
2) "12345678"

4.appendonly yes  #开启AOF模式
5.save配置解读：

1) “save 900 1”表示如果900秒内至少1个key发生变化（新增、修改和删除），则重写rdb文件；

2) “save 300 10”表示如果每300秒内至少10个key发生变化（新增、修改和删除），则重写rdb文件；

3) “save 60 3600”表示如果每60秒内至少10000个key发生变化（新增、修改和删除），则重写rdb文件。

作用：

控制什么时候生成rdb文件（快照，也可叫Checkpoint，即检查点）。

进程启动的时候，会将每一行save读进到类型为struct saveparam的数组中。这个没有排序，依在redis.conf中的先后顺序。在检查时，只要满足就不会再检查下一条规则。

配置策略：

如果同时开启了aof，则可考虑将save的参数调大一点，以减少写rdb带来的压力。实际上如果开启了aof，redis在启动时只会读取aof文件，而不会读取rdb文

更多解读参考文档：
https://www.cnblogs.com/wiseleer/p/16905980.html

## 三、编写 docker-compose.yaml文件

```sh
[root@iZ2ze505h9bgsbp83ct28pZ redis]# cd /usr/local/docker/redis
[root@iZ2ze505h9bgsbp83ct28pZ redis]# vim docker-compose.yml
```
写入下面的内容

```sh
version: '3.8'
services:
  myredis:
    container_name: myredis
    image: redis:6.0.5 #redis:6.2.6
    restart: always
    ports:
      - 6379:6379
    privileged: true
    environment:
      # 时区上海
      TZ: Asia/Shanghai
    command: redis-server /etc/redis/redis.conf --appendonly yes
    volumes:
      - $PWD/data:/data
      - $PWD/conf/redis.conf:/etc/redis/redis.conf
    networks:
      - myweb

networks:

  myweb:
    driver: bridge
```

说明:
/etc/redis/redis.conf为容器里的目录
$PWD/conf/redis.conf 为宿主机里的目录

启动前的目录结构

```sh
root@haima-PC:/usr/local/docker/redis# tree
.
├── conf
│   └── redis.conf
└── docker-compose.yml

```

## 四、启动容器

```sh
[root@iZ2ze505h9bgsbp83ct28pZ redis]# docker-compose up -d # 后台启动
[root@iZ2ze505h9bgsbp83ct28pZ redis]# docker ps -a #查看启动的容器
CONTAINER ID        IMAGE                    COMMAND                  CREATED             STATUS                    PORTS                                                NAMES
f0970e970457        redis                    "docker-entrypoint.s…"   4 minutes ago       Up 4 minutes              0.0.0.0:6379->6379/tcp                               myredis
[root@iZ2ze505h9bgsbp83ct28pZ redis]# docker exec -it myredis redis-cli # 启动redis客户连接redis服务 
127.0.0.1:6379> keys *
(error) NOAUTH Authentication required.
127.0.0.1:6379> auth 123456 # 输入密码登陆
OK
127.0.0.1:6379> set name test
OK
127.0.0.1:6379> get name 
"test"

```

到这里就ok了，

启动后的目录结构

```sh
root@haima-PC:/usr/local/docker/redis# tree
.
├── conf
│   └── redis.conf
├── data
│   ├── appendonly.aof
│   └── dump.rdb
└── docker-compose.yml

```

## 五、限制内在和CPU

如果想要限制内在和CPU

1.2加入deploy的resource limit

```sh
version: '3.8'
services:
  myredis:
    container_name: myredis
    image: redis:6.0.6 #redis:6.0.6
    restart: always
    ports:
      - 6379:6379
    privileged: true
    environment:
      # 时区上海
      TZ: Asia/Shanghai
    command: redis-server /etc/redis/redis.conf --appendonly yes
    volumes:
      - $PWD/data:/data
      - $PWD/conf/redis.conf:/etc/redis/redis.conf
    deploy:
      resources:
        limits:
          cpus: '0.50'
          memory: 500M
    networks:
      - myweb
      

networks:

  myweb:
    driver: bridge

```

2.启动container时加入--compatibility参数

  `docker-compose --compatibility up -d`

2.1启动时未加入-compatibility的警告

  `WARNING: Some services（redis）use the 'deploy' key，which will be ignored.Compose does not support 'deploy' configuration - use `docker stack deploy` to deploy to a swarm.`


查看redis的CPU和内存使用状态

```sh
[root@iZ2ze505h9bgsbp83ct28pZ redis]# docker stats myredis
```

```sh
CONTAINER ID        NAME                CPU %               MEM USAGE / LIMIT   MEM %               NET I/O             BLOCK I/O           PIDS
2cb96f4cfcf9        myredis             0.10%               9.59MiB / 500MiB    1.92%               2.69kB / 0B         0B / 0B             5
```

## 六、启用redis-cli

新开一个命令行窗口

```sh
# 客户端连接redis 无密码方式
[root@middle ~]# docker exec -it myredis redis-cli

# 客户端连接redis 123456是密码
[root@middle ~]# docker exec -it myredis redis-cli -a 123456

# 或者 使用auth命令输入密码
[root@middle ~]# docker exec -it myredis redis-cli
127.0.0.1:6379> auth 123456
OK

127.0.0.1:6379> get name
(nil)
127.0.0.1:6379> set name lisi
OK
127.0.0.1:6379> get name
"lisi"
```

## 七、查看容器常用的命令
```sh
docker ps -a  # 查看所有容器
docker ps # 查看正在运行的容器
docker restart container-id # 重启某个容器id或者容器名字
docker stop container-id # 关闭指定容器
docker rm myredis  #移除指定容器
docker start 容器id //启动容器记录里的容器
docker rm container-id # 删除关闭的容器
docker rm -f container-id # 关闭并删除关闭的容器
docker rm $(docker ps -a -q)  #移除掉所有容器
```